from pathlib import Path
from typing import Optional

import typer
from django.apps import apps

from auto_ninja.cases import snake_case, kebab_case
from auto_ninja.django_services import (
    modify_django_urls,
    get_settings_module,
    setup_django_environment,
)
from auto_ninja.services import find_auto_ninja_templates, file, folder


def main(
    title: Optional[str] = "Auto Ninja",
    description: Optional[str] = "Automatically generated API",
):  # Variables
    settings_module = get_settings_module()
    settings = Path(settings_module.replace(".", "/")).parent
    setup_django_environment(settings_module)
    templates = find_auto_ninja_templates()
    installed_apps = [(Path(app.name), app) for app in apps.get_app_configs() if Path(app.name).exists()]

    # Settings
    api = folder(settings / "api")
    file(api / "__init__.py")
    file(api / "services.py/", templates / "services.py.txt")
    file(api / "utils.py/", templates / "utils.py.txt")
    file(
        api / "main.py/",
        templates / "main.py.txt",
        {"apps": installed_apps, "title": title, "description": description},
    )
    modify_django_urls(settings / "urls.py")

    # Apps
    for app_path, app in installed_apps:
        app_api = folder(app_path / "api")
        file(app_api / "__init__.py")
        endpoints = folder(app_api / "endpoints")
        schemas = folder(app_api / "schemas")

        models = [model.__name__ for model in app.get_models()]
        model_modules = [snake_case(model) for model in models]
        model_paths_modules = [(kebab_case(model), snake_case(model)) for model in models]

        file(
            endpoints / "__init__.py",
            templates / "endpoint_init.py.txt",
            {
                "path_modules": model_paths_modules,
                "app": app,
                "settings": str(settings),
            },
        )
        file(
            schemas / "__init__.py",
            templates / "schema_init.py.txt",
            {"models": model_modules},
        )

        for model in models:
            context = {
                "app": app,
                "model": model,
                "settings": str(settings),
            }
            file(
                endpoints / f"{snake_case(model)}.py",
                templates / "endpoint.py.txt",
                context,
            )
            file(
                schemas / f"{snake_case(model)}.py",
                templates / "schema.py.txt",
                context,
            )


if __name__ == "__main__":
    typer.run(main)



import importlib
from pathlib import Path

from django.template import Engine, Context


def folder(dest: Path) -> Path:
    dest.mkdir(parents=True, exist_ok=True)
    return dest


def file(dest: Path, template_file: Path = None, context: dict = None):
    context = context or {}
    if template_file:
        template = Engine(dirs=[], app_dirs=True).from_string(template_file.read_text())
        dest.write_text(template.render(Context(context)))
    else:
        dest.touch(exist_ok=True)


def find_auto_ninja_templates() -> Path:
    spec = importlib.util.find_spec("auto_ninja")
    if spec is None or spec.origin is None:
        raise ImportError("auto_ninja package not found")
    package_path = Path(spec.origin).parent
    return package_path / "templates"



import ast
import os
import re
import sys
from pathlib import Path

import django

from auto_ninja.services import file


def extract_django_settings_module(manage_py_path: Path) -> str:
    content = manage_py_path.read_text()

    pattern = r"os\.environ\.setdefault\(\s*['\"]DJANGO_SETTINGS_MODULE['\"]\s*,\s*['\"](?P<settings>[\w\.]+)['\"]\s*\)"
    match = re.search(pattern, content)

    if not match:
        raise ValueError("DJANGO_SETTINGS_MODULE not found in manage.py")
    return match.group("settings")


def get_settings_module():
    manage_py_path = Path("manage.py")
    return extract_django_settings_module(manage_py_path)


def setup_django_environment(settings_module: str):
    file(Path("__init__.py"))
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", settings_module)
    sys.path.insert(0, os.getcwd())
    django.setup()


class ImportInjector(ast.NodeTransformer):
    def __init__(self, urls_path: Path):
        self.urls_path = urls_path
        self.import_found = False
        self.urlpatterns_found = False

    def visit_ImportFrom(self, node):
        # Check if the import already exists
        if node.module == f"{self.urls_path.parent.name}.api.main" and any(alias.name == "api" for alias in node.names):
            self.import_found = True
        return node

    def visit_Assign(self, node):
        # Look for the 'urlpatterns' assignment
        if any(isinstance(target, ast.Name) and target.id == "urlpatterns" for target in node.targets):
            # Check if 'api/' path is already present
            for elt in node.value.elts:
                if (
                    isinstance(elt, ast.Call)
                    and len(elt.args) > 0
                    and (isinstance(elt.args[0], ast.Constant) and elt.args[0].value == "api/")
                ):
                    self.urlpatterns_found = True
                    break

            # If not found, add it to the urlpatterns list
            if not self.urlpatterns_found:
                new_call = ast.Call(
                    func=ast.Name(id="path", ctx=ast.Load()),
                    args=[
                        ast.Constant(value="api/"),
                        ast.Attribute(
                            value=ast.Name(id="api", ctx=ast.Load()),
                            attr="urls",
                            ctx=ast.Load(),
                        ),
                    ],
                    keywords=[],
                )
                node.value.elts.append(new_call)
                self.urlpatterns_found = True
        return node


def modify_django_urls(urls_path: Path):
    tree = ast.parse(urls_path.read_text())

    injector = ImportInjector(urls_path)
    injector.visit(tree)

    # Add the import statement if it wasn't found
    if not injector.import_found:
        new_import = ast.ImportFrom(
            module=f"{urls_path.parent.name}.api.main",
            names=[ast.alias(name="api", asname=None)],
            level=0,
        )
        # Insert at the start, after initial module docstring if any
        if isinstance(tree.body[0], ast.Expr) and isinstance(tree.body[0].value, ast.Constant):
            tree.body.insert(1, new_import)
        else:
            tree.body.insert(0, new_import)

    # Write changes back to the file using ast.unparse
    urls_path.write_text(ast.unparse(tree))
